@file:Suppress("KDocUnresolvedReference")

package net.sergeych.ugdx.shared

import net.sergeych.boss_serialization_mp.KVBinaryStorage
import net.sergeych.bossk.Bossk
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.debug
import net.sergeych.mp_logger.exception
import java.io.File
import java.nio.file.Files
import java.nio.file.Paths

class SimpleFileStorage(private val file: File) : KVBinaryStorage, LogTag("SBSTR") {
    private val storage: HashMap<String,ByteArray> = run {
        try {
            Bossk.unpack<HashMap<String,ByteArray>>(file.readBytes()).also {
                debug { "storage loaded" }
            }
        }
        catch(t: Throwable) {
            exception {  "Failed to load params from $file" to t }
            HashMap()
        }
    }

    override val keys: Set<String>
        get() = storage.keys

    override fun get(key: String): ByteArray?
        = storage.get(key)

    override fun remove(key: String): ByteArray? =
        storage.remove(key)?.also {
            save()
        }

    override fun set(key: String, value: ByteArray) {
        storage.set(key,value)
        save()
    }

    override fun save() {
        file.writeBytes(Bossk.pack(storage))
        debug { "storgae saved: $file" }
    }

}

/**
 * Create a platform-specific permanent binary storage, suitable to use with GuestAPI. Just pass it
 * to [GusetAPI.setup]:
 * ~~~
 * GuestAPI.setup {
 *   storage = PermanentBinaryStorage("mayAppName")
 * }
 * ~~~
 * Note that the storage itself is not encrypted, see [EncryptedPermanentBinaryStorage] for a safe version
 */
actual fun PermanentBinaryStorage(name: String): KVBinaryStorage {
    val root = "${System.getProperty("user.home")}/.ugdx/sessions"
    Files.createDirectories(Paths.get(root))
    return SimpleFileStorage(File(root + "$name.boss"))
}